@@ -6,8 +6,9 @@ self    <- 'shut up R CMD CHECK'
 private <- 'shut up R CMD CHECK'
 not_implemented <- function() stop('Currently not implemented')
 
-initialize <- function(token) {
+initialize <- function(token, proxy) {
     self$set_token(token)
+    self$set_proxy(proxy)
 }
 
 set_token <- function(token){
@@ -15,6 +16,26 @@ set_token <- function(token){
         private$token <- token
 }
 
+set_proxy <- function(proxy){
+    if (!missing(proxy)){
+        if (is.list(proxy)){
+            private$proxy_url      <- as.character(proxy$url)
+            private$proxy_port     <- as.integer(proxy$port)
+            private$proxy_username <- as.character(proxy$username)
+            private$proxy_password <- as.character(proxy$password)
+            private$proxy_auth     <-
+                if ("auth" %in% names(proxy)) as.character(proxy$auth)
+                else 'basic'
+            private$proxy_set      <- TRUE
+        } else {
+            private$proxy_set <- FALSE
+        }
+    } else {
+        private$proxy_set <- FALSE
+    }
+}
+
+
 set_default_chat_id <- function(chat_id){
     if (!missing(chat_id))
         private$default_chat_id <- as.character(chat_id)
@@ -33,19 +54,32 @@ request <- function(method = NULL, body = NULL){
                        method)
     private$lr_method <- method
     private$lr_body <- body
-    private$lr_response <- r <- httr::POST(url = api_url, body = body)
+    private$lr_response <- r <-
+        if (private$proxy_set) {
+            ## use proxy
+            httr::POST(url = api_url,
+                       body = body,
+                       httr::use_proxy(
+                                 url = private$proxy_url,
+                                 port = private$proxy_port,
+                                 username = private$proxy_username,
+                                 password = private$proxy_password,
+                                 auth = private$proxy_auth))
+        } else {
+            ## without proxy
+            httr::POST(url = api_url, body = body)
+        }
     httr::warn_for_status(r)
     r
 }
 
 last_request <- function(){
-
     list('method'   = private$lr_method,
          'body'     = private$lr_body,
          'response' = private$lr_response)
-
 }
 
+## pretty printer for methods
 make_methods_string <- function(meth, incipit){
     wrap_at <- 72
     meth_string <- paste0(incipit, '\n',  paste(meth, collapse = ", "))
@@ -65,11 +99,12 @@ tgprint <- function(){
                      "sendSticker",
                      "sendVideo",
                      "sendVoice",
+                     'sendPoll',
                      "sendLocation",
                      "sendChatAction",
                      "getUserProfilePhotos",
                      "getUpdates",
-                     "setWebhook",
+                     ## "setWebhook",
                      "getFile")
     dont_show <- c("clone", "initialize", "print")
     avail_methods <- sort(api_methods[api_methods %in% obj])
@@ -97,7 +132,13 @@ check_chat_id <- function(chat_id){
         return(chat_id)
 }
 
-check_param <- function(param, type, required = FALSE){
+## check for user insertion for a method
+## - param is the insertion
+## - type is data type
+## - required is if the parameter is required for the method
+## - if scalar is TRUE take only the first element (of the object, eg
+##   the vector)
+check_param <- function(param, type, required = FALSE, scalar = TRUE){
     char_name <- deparse(substitute(char))
     coerce <- c('char'      = as.character,
                 'int'       = as.integer,
@@ -106,8 +147,13 @@ check_param <- function(param, type, required = FALSE){
     if(is.null(param)){
         if (required) stop(char_name, " can't be missing.")
         else NULL
+    } else {
+        ## scalar by default (but if set to false leave a vector of
+        ## parameters (eg used in sendPoll for answers)
+        rval <- coerce[[type]](param)
+        if (scalar) rval <- rval[1]
+        rval
     }
-    else coerce[[type]](param[1])
 }
 
 check_file <- function(path, required = FALSE){
@@ -133,22 +179,28 @@ parsed_content <- function(x){
 #' forwardMessage
 #'
 #' Forward messages of any kind
+#' 
 #' @param from_chat_id Unique identifier for the chat where the
 #'     original message was sent (required)
 #' @param message_id Unique message identifier (required)
+#' @param disable_notification Sends the message silently. Users will
+#'     receive a notification with no sound
 #' @param chat_id Unique identifier for the target chat or username of
 #'     the target channel (required)
 forwardMessage <- function(from_chat_id = NULL,
                            message_id = NULL,
+                           disable_notification = NULL,
                            chat_id = NULL)
 {
     ## params
     chat_id <- private$check_chat_id(chat_id = chat_id)
     from_chat_id <- check_param(from_chat_id, 'char', required = TRUE)
     message_id <- check_param(message_id, 'char', required = TRUE)
+    disable_notification <- check_param(disable_notification, 'log')
     ## request body
     body <- make_body('chat_id' = chat_id,
                       'from_chat_id' = from_chat_id,
+                      'disable_notification' = disable_notification,
                       'message_id' = message_id)
     ## request
     r <- private$request('forwardMessage', body = body)
@@ -159,6 +211,7 @@ forwardMessage <- function(from_chat_id = NULL,
 #' getFile
 #'
 #' Get info about a file and download it
+#'
 #' @param file_id File identifier (required)
 #' @param destfile Destination path; if specified the file will be
 #'     downloaded
@@ -201,8 +254,8 @@ getMe <- function()
 #' getUpdates
 #'
 #' Receive incoming updates
-#' @param offset Identifier of the first update to be returned
-#'     returned.
+#' 
+#' @param offset Identifier of the first update to be returned.
 #' @param limit Limits the number of updates to be retrieved. Values
 #'     between 1-100 are accepted. Defaults to 100
 getUpdates <- function(offset = NULL,
@@ -226,17 +279,15 @@ getUpdates <- function(offset = NULL,
 #' getUserProfilePhotos
 #'
 #' Get a list of profile pictures for a user
+#' 
 #' @param user_id Unique identifier of the target user (required)
 #' @param offset Sequential number of the first photo to be
 #'     returned. By default, all photos are returned
 #' @param limit Limits the number of photos to be retrieved. Values
 #'     between 1-100 are accepted. Defaults to 100
-#' @param destfile if a path is specified save the image (by default
-#'     the bigger) in a local file
 getUserProfilePhotos <- function(user_id = NULL,
                                  offset = NULL,
-                                 limit = NULL,
-                                 destfile = NULL)
+                                 limit = NULL)
 {
     ## params
     user_id <- check_param(user_id, 'int', required = TRUE)
@@ -252,51 +303,54 @@ getUserProfilePhotos <- function(user_id = NULL,
     if (r$status == 200){
         file_id <- parsed_content(r)$photos
         rval <- do.call(rbind, file_id)
-        if (!is.null(destfile)){
-            path <- rval$file_path
-            path <- path[!is.na(path)]
-            dl_url <- sprintf('https://api.telegram.org/file/bot%s/%s',
-                              private$token,
-                              path)
-            curl::curl_download(dl_url, destfile = destfile)
-            invisible(rval)
-        } else
-            return(rval)
+        return(rval)
     } else
         invisible(NULL)
 }
 
+
+
 #' sendAudio
 #'
 #' Send \code{mp3} files
+#' 
 #' @param audio path to audio file to send (required)
 #' @param duration duration of the audio in seconds
 #' @param performer performer
 #' @param title track name
+#' @param caption audio caption
+#' @param disable_notification Sends the message silently. Users will
+#'     receive a notification with no sound
 #' @param reply_to_message_id If the message is a reply, ID of the
 #'     original message
 #' @param chat_id Unique identifier for the target chat or username of
 #'     the target channel (required)
 sendAudio <- function(audio = NULL,
+                      caption = NULL,
                       duration = NULL,
                       performer = NULL,
                       title = NULL,
+                      disable_notification = NULL,
                       reply_to_message_id = NULL,
                       chat_id = NULL)
 {
     ## params
     chat_id <- private$check_chat_id(chat_id = chat_id)
+    caption <- check_param(caption, 'char')
     audio <- check_file(audio, required = TRUE)
     duration <- check_param(duration, 'int')
     performer <- check_param(performer, 'char')
     title <- check_param(title, 'char')
+    disable_notification <- check_param(disable_notification, 'log')
     reply_to_message_id <- check_param(reply_to_message_id, 'int')
     ## request body
     body <- make_body('chat_id' = chat_id,
                       'audio' = httr::upload_file(audio),
+                      'caption' = caption,
                       'duration' = duration,
                       'performer' = performer,
                       'title' = title,
+                      'disable_notification' = disable_notification,
                       'reply_to_message_id' = reply_to_message_id)
     ## request
     r <- private$request('sendAudio', body = body)
@@ -304,27 +358,58 @@ sendAudio <- function(audio = NULL,
     invisible(r)
 }
 
-sendChatAction <- function() not_implemented()
+#' sendChatAction
+#'
+#' Tell the user that something is happening on the bot's side
+#' @param action type of action to broadcast. Choose one, depending on
+#'     what the user is about to receive: "typing" for text messages,
+#'     "upload_photo" for photos, "record_video" or "upload_video" for
+#'     videos, "record_voice" or "upload_voice" for voice notes,
+#'     "upload_document" for general files, "find_location" for
+#'     location data, "record_video_note" or "upload_video_note" for
+#'     video notes.
+#' @param chat_id Unique identifier for the target chat or username of
+#'     the target channel (required)
+sendChatAction <- function(action = NULL,
+                           chat_id = NULL)
+{
+    ## params
+    chat_id <- private$check_chat_id(chat_id = chat_id)
+    action <- check_param(action, 'char')
+    ## request body
+    body <- make_body('chat_id' = chat_id,
+                      'action' = action)
+    ## request
+    r <- private$request('sendChatAction', body = body)
+    ## response handling
+    invisible(r)
+}
 
 #' sendDocument
 #'
 #' Send general files
+#' 
 #' @param document path to the file to send (required)
+#' @param disable_notification Sends the message silently. Users will
+#'     receive a notification with no sound
 #' @param reply_to_message_id if the message is a reply, ID of the
 #'     original message
 #' @param chat_id Unique identifier for the target chat or username of
 #'     the target channel (required)
 sendDocument <- function(document = NULL,
+                         disable_notification = NULL,
                          reply_to_message_id = NULL,
                          chat_id = NULL)
 {
     ## params
     chat_id <- private$check_chat_id(chat_id = chat_id)
     document <- check_file(document, required = TRUE)
+    disable_notification <- check_param(disable_notification, 'log')
     reply_to_message_id <- check_param(reply_to_message_id, 'int')
     ## request body
     body <- make_body('chat_id' = chat_id,
                       'document' = httr::upload_file(document),
+                      'disable_notification' = disable_notification,
                       'reply_to_message_id' = reply_to_message_id)
     ## request
     r <- private$request('sendDocument', body = body)
@@ -335,14 +420,18 @@ sendDocument <- function(document = NULL,
 #' sendLocation
 #'
 #' Send point on the map
+#' 
 #' @param latitude Latitude of location (required)
 #' @param longitude Longitude of location (required)
+#' @param disable_notification Sends the message silently. Users will
+#'     receive a notification with no sound
 #' @param reply_to_message_id If the message is a reply, ID of the
 #'     original message
 #' @param chat_id Unique identifier for the target chat or username of
 #'     the target channel (required)
 sendLocation <- function(latitude = NULL,
                          longitude = NULL,
+                         disable_notification = NULL,
                          reply_to_message_id = NULL,
                          chat_id = NULL)
 {
@@ -350,11 +439,13 @@ sendLocation <- function(latitude = NULL,
     chat_id <- private$check_chat_id(chat_id = chat_id)
     latitude <- check_param(latitude, 'float', required = TRUE)
     longitude <- check_param(longitude, 'float', required = TRUE)
+    disable_notification <- check_param(disable_notification, 'log')
     reply_to_message_id <- check_param(reply_to_message_id, 'int')
     ## request body
     body <- make_body('chat_id' = chat_id,
                       'latitude' = latitude,
                       'longitude' = longitude,
+                      'disable_notification' = disable_notification,
                       'reply_to_message_id' = reply_to_message_id)
     ## request
     r <- private$request('sendLocation', body = body)
@@ -365,11 +456,14 @@ sendLocation <- function(latitude = NULL,
 #' sendMessage
 #'
 #' Send text messages
+#' 
 #' @param text Text of the message to be sent (required)
 #' @param parse_mode send 'Markdown' if you want Telegram apps to show
 #'     bold, italic and inline URLs in your bot's message
 #' @param disable_web_page_preview Disables link previews for links in
 #'     this message
+#' @param disable_notification Sends the message silently. Users will
+#'     receive a notification with no sound
 #' @param reply_to_message_id If the message is a reply, ID of the
 #'     original message
 #' @param chat_id Unique identifier for the target chat or username of
@@ -377,6 +471,7 @@ sendLocation <- function(latitude = NULL,
 sendMessage <- function(text = NULL,
                         parse_mode = NULL,
                         disable_web_page_preview = NULL,
+                        disable_notification = NULL,
                         reply_to_message_id = NULL,
                         chat_id = NULL)
 {
@@ -385,11 +480,14 @@ sendMessage <- function(text = NULL,
     text <- check_param(text, 'char', required = TRUE)
     parse_mode <- check_param(parse_mode, 'char')
     disable_web_page_preview <- check_param(disable_web_page_preview, 'log')
+    disable_notification <- check_param(disable_notification, 'log')
     reply_to_message_id <- check_param(reply_to_message_id, 'int')
     ## request body
     body <- make_body('chat_id' = chat_id,
                       'text' = as.character(text),
                       'parse_mode' = parse_mode,
+                      'disable_web_page_preview' = disable_web_page_preview,
+                      'disable_notification' = disable_notification,
                       'reply_to_message_id' = reply_to_message_id)
     ## request
     r <- private$request('sendMessage', body = body)
@@ -400,14 +498,18 @@ sendMessage <- function(text = NULL,
 #' sendPhoto
 #'
 #' Send image files
+#' 
 #' @param photo photo to send (required)
 #' @param caption photo caption
+#' @param disable_notification Sends the message silently. Users will
+#'     receive a notification with no sound
 #' @param reply_to_message_id If the message is a reply, ID of the
 #'     original message
 #' @param chat_id Unique identifier for the target chat or username of
 #'     the target channel (required)
 sendPhoto <- function(photo = NULL,
                       caption = NULL,
+                      disable_notification = NULL,
                       reply_to_message_id = NULL,
                       chat_id = NULL)
 {
@@ -415,11 +517,13 @@ sendPhoto <- function(photo = NULL,
     chat_id <- private$check_chat_id(chat_id = chat_id)
     photo <- check_file(photo, required = TRUE)
     caption <- check_param(caption, 'char')
+    disable_notification <- check_param(disable_notification, 'log')
     reply_to_message_id <- check_param(reply_to_message_id, 'int')
     ## request body
     body <- make_body('chat_id' = chat_id,
                       'photo' = httr::upload_file(photo),
                       'caption' = caption,
+                      'disable_notification' = disable_notification,
                       'reply_to_message_id' = reply_to_message_id)
     ## request
     r <- private$request('sendPhoto', body = body)
@@ -430,22 +534,28 @@ sendPhoto <- function(photo = NULL,
 #' sendSticker
 #'
 #' Send \code{.webp} stickers
+#' 
 #' @param sticker sticker to send (required)
+#' @param disable_notification Sends the message silently. Users will
+#'     receive a notification with no sound
 #' @param reply_to_message_id If the message is a reply, ID of the
 #'     original message
 #' @param chat_id Unique identifier for the target chat or username of
 #'     the target channel (required)
 sendSticker <- function(sticker = NULL,
+                        disable_notification = NULL,
                         reply_to_message_id = NULL,
                         chat_id = NULL)
 {
     ## params
     chat_id <- private$check_chat_id(chat_id = chat_id)
     sticker <- check_file(sticker, required = TRUE)
+    disable_notification <- check_param(disable_notification, 'log')
     reply_to_message_id <- check_param(reply_to_message_id, 'int')
     ## request body
     body <- make_body('chat_id' = chat_id,
                       'sticker' = httr::upload_file(sticker),
+                      'disable_notification' = disable_notification,
                       'reply_to_message_id' = reply_to_message_id)
     ## request
     r <- private$request('sendSticker', body = body)
@@ -456,6 +566,7 @@ sendSticker <- function(sticker = NULL,
 #' sendVideo
 #'
 #' Send \code{mp4} videos
+#' 
 #' @param video Video to send (required)
 #' @param duration Duration of sent video in seconds
 #' @param caption Video caption
@@ -490,14 +601,18 @@ sendVideo <- function(video = NULL,
 #' sendVoice
 #'
 #' Send \code{.ogg} files encoded with OPUS
+#' 
 #' @param voice Audio file to send (required)
 #' @param duration Duration of sent audio in seconds
+#' @param disable_notification Sends the message silently. Users will
+#'     receive a notification with no sound
 #' @param reply_to_message_id If the message is a reply, ID of the
 #'     original message
 #' @param chat_id Unique identifier for the target chat or username of
 #'     the target channel (required)
 sendVoice <- function(voice = NULL,
                       duration = NULL,
+                      disable_notification = NULL,
                       reply_to_message_id = NULL,
                       chat_id = NULL)
 {
@@ -505,11 +620,13 @@ sendVoice <- function(voice = NULL,
     chat_id <- private$check_chat_id(chat_id = chat_id)
     voice <- check_file(voice, required = TRUE)
     duration <- check_param(duration, 'int')
+    disable_notification <- check_param(disable_notification, 'log')
     reply_to_message_id <- check_param(reply_to_message_id, 'int')
     ## request body
     body <- make_body('chat_id' = chat_id,
                       'voice' = httr::upload_file(voice),
                       'duration' = duration,
+                      'disable_notification' = disable_notification,
                       'reply_to_message_id' = reply_to_message_id)
     ## request
     r <- private$request('sendVoice', body = body)
@@ -517,7 +634,122 @@ sendVoice <- function(voice = NULL,
     invisible(r)
 }
 
-setWebhook <- function() not_implemented()
+#' sendDice
+#'
+#' Use this method to send a dice, which will have a random value from
+#' 1 to 6.
+#'
+#' @param disable_notification Sends the message silently. Users will
+#'     receive a notification with no sound
+#' @param chat_id Unique identifier for the target chat or username of
+#'     the target channel (required)
+sendDice <- function(disable_notification = NULL,
+                     chat_id = NULL)
+{
+    ## params
+    chat_id <- private$check_chat_id(chat_id = chat_id)
+    disable_notification <- check_param(disable_notification, 'log')
+    ## request body
+    body <- make_body(
+        'chat_id' = chat_id,
+        'disable_notification' = disable_notification)
+    ## request
+    r <- private$request('sendDice', body = body)
+    ## response handling
+    invisible(r)
+}
+
+#' sendPoll
+#'
+#' Use this method to send a native poll.
+#' 
+#' @param question Poll question, 1-255 characters
+#' @param options vector of possible answers: 2-10 strings 1-100
+#'     characters each
+#' @param is_anonymous if TRUE (default) the poll needs to be
+#'     anonymous
+#' @param type poll type, 'quiz' or 'regular', defaults to 'regular'
+#' @param allows_multiple_answers if TRUE the poll allows multiple
+#'     answers (default to FALSE and ignored for polls in quiz mode)
+#' @param correct_option_id 0-based identifier of the correct answer
+#'     option, required for polls in quiz mode (eg 0 set the first
+#'     answer as correct)
+#' @param is_closed if TRUE the poll needs to be immediately
+#'     closed. This can be useful for poll preview. (default to FALSE)
+#' @param disable_notification Sends the message silently. Users will
+#'     receive a notification with no sound
+#' @param reply_to_message_id If the message is a reply, ID of the
+#'     original message
+#' @param chat_id Unique identifier for the target chat or username of
+#'     the target channel (required)
+#' 
+sendPoll <- function(question = NULL,
+                     options = NULL,
+                     is_anonymous = TRUE,
+                     type = c('regular', 'quiz'),
+                     allows_multiple_answers = FALSE,
+                     correct_option_id = NULL,
+                     is_closed = FALSE,
+                     disable_notification = NULL,
+                     reply_to_message_id = NULL,
+                     chat_id = NULL)
+{
+    ## params
+    chat_id <- private$check_chat_id(chat_id = chat_id)
+    question <- check_param(question, 'char')
+    options <- check_param(options, 'char', scalar = FALSE)
+    is_anonymous <- check_param(is_anonymous, 'log')
+    type <- match.arg(type)
+    allows_multiple_answers <- check_param(allows_multiple_answers, 'log')
+    ## correct options is required only for quiz polls
+    correct_option_id <- check_param(
+        correct_option_id, 'int', required = type %in% 'quiz')
+    is_closed <- check_param(is_closed, 'log')
+    disable_notification <- check_param(disable_notification, 'log')
+    reply_to_message_id <- check_param(reply_to_message_id, 'int')
+    ## request body
+    body <- make_body('chat_id' = chat_id,
+                      'question' = question,
+                      'options' = jsonlite::toJSON(options),
+                      'is_anonymous' = is_anonymous,
+                      'type' = type,
+                      'allows_multiple_answers' = allows_multiple_answers,
+                      'correct_option_id' = correct_option_id,
+                      'disable_notification' = disable_notification,
+                      'is_closed' = is_closed,
+                      'reply_to_message_id' = reply_to_message_id)
+    ## request
+    r <- private$request('sendPoll', body = body)
+    ## response handling
+    invisible(r)
+}
+
+
+#' stopPoll
+#'
+#' Use this method to stop a poll which was sent by the bot.
+#' 
+#' @param message_id Identifier of the original message with the poll
+#' @param chat_id Unique identifier for the target chat or username of
+#'     the target channel (required)
+stopPoll <- function(message_id = NULL,
+                     chat_id = NULL)
+{
+    ## params
+    chat_id <- private$check_chat_id(chat_id = chat_id)
+    ## message_id <- check_param(message_id, 'int', required = TRUE)
+    
+    ## request body
+    body <- make_body('chat_id' = chat_id,
+                      'message_id' = message_id)
+    ## request
+    r <- private$request('stopPoll', body = body)
+    ## response handling
+    invisible(r)
+}
+
+
+## setWebhook <- function() not_implemented()
 
 
 #' TGBot
@@ -525,7 +757,7 @@ setWebhook <- function() not_implemented()
 #' Package main class (implementing the Telegram bot).
 #' 
 #' @docType class
-#' @format An \code{\link{R6Class}} generator object.
+#' @format An \code{\link[R6]{R6Class}} generator object.
 #' @section API Methods: \describe{
 #'     \item{\code{\link{forwardMessage}}}{forward messages of any
 #'     kind} \item{\code{\link{getFile}}}{get info about a file and
@@ -541,12 +773,27 @@ setWebhook <- function() not_implemented()
 #'     \item{\code{\link{sendSticker}}}{send \code{.webp} stickers}
 #'     \item{\code{\link{sendVideo}}}{send \code{mp4} videos}
 #'     \item{\code{\link{sendVoice}}}{send ogg files encoded with
-#'     OPUS} }
+#'     OPUS} 
+#'     \item{\code{\link{sendPoll}}}{send a telegram poll}
+#'     \item{\code{\link{stopPoll}}}{stop a telegram poll and obtain results}
+#' }
 #' @references \href{http://core.telegram.org/bots}{Bots: An
 #'     introduction for developers} and
 #'     \href{http://core.telegram.org/bots/api}{Telegram Bot API}
 #' @examples \dontrun{
+#' 
+#' ## Without proxy
 #' bot <- TGBot$new(token = bot_token('RBot'))
+#' 
+#' ## With a proxy ...
+#' prx <- list('url' = '123.45.6.78',
+#'             'port' = 8080,
+#'             'username' = 'user',
+#'             'password' = 'password')
+#' bot <- TGBot$new(token = bot_token('RBot'), proxy = prx)
+#' ##  .. or 
+#' bot <- TGBot$new(token = bot_token('RBot'))
+#' bot$set_proxy(proxy = prx)
 #' }
 #' @export
 TGBot <- R6::R6Class("TGBot",
@@ -556,14 +803,32 @@ TGBot <- R6::R6Class("TGBot",
                          ## ---------------------
                          initialize = initialize,
                          set_token = set_token,
+                         set_proxy = set_proxy,
                          set_default_chat_id = set_default_chat_id,
                          print = tgprint,
-                         last_request = last_request, ## for debug only,
-                                                      ## comment on release!
+
+                         ## This is for development/debug only, comment on release!
+                         req = request,
+                         last_request = last_request,
 
                          ## ---------------------
                          ## methods - TG api
                          ## ---------------------
+                         
+                         ## later or never
+                         ## --------------
+                         ## sendAnimation 
+                         ## sendMediaGroup
+                         ## sendVideoNote
+                         ## **LiveLocation
+                         ## sendChatAction
+                         ## sendVenue
+                         ## sendContact
+                         ## **Chat*
+                         ## answerCallbackQuery
+                         ## setMyCommands
+                         ## other **Sticker** stuff
+                         
                          forwardMessage       = forwardMessage,
                          getFile              = getFile,
                          getMe                = getMe,
@@ -571,14 +836,18 @@ TGBot <- R6::R6Class("TGBot",
                          getUserProfilePhotos = getUserProfilePhotos,
                          sendAudio            = sendAudio,
                          sendChatAction       = sendChatAction,
+                         sendDice             = sendDice,
                          sendDocument         = sendDocument,
                          sendLocation         = sendLocation,
                          sendMessage          = sendMessage,
                          sendPhoto            = sendPhoto,
+                         sendPoll             = sendPoll,
                          sendSticker          = sendSticker,
                          sendVideo            = sendVideo,
                          sendVoice            = sendVoice,
-                         setWebhook           = setWebhook
+                         ## setWebhook           = setWebhook,
+                         stopPoll             = stopPoll
+                         
                      ),
                      private = list(
                          ## ---------------------
@@ -588,9 +857,17 @@ TGBot <- R6::R6Class("TGBot",
                          default_chat_id = NULL,
                          bot_first_name = NULL,
                          bot_username = NULL,
-                         lr_method = NULL,    ## last requested method
-                         lr_body = NULL,      ## last request's body
-                         lr_response = NULL,  ## last request's response
+                         ## proxy data
+                         proxy_set      = NULL,
+                         proxy_url      = NULL,
+                         proxy_port     = NULL,
+                         proxy_username = NULL,
+                         proxy_password = NULL,
+                         proxy_auth     = NULL,
+                         ## last request (LR) infos
+                         lr_method = NULL,    ## method
+                         lr_body = NULL,      ## body
+                         lr_response = NULL,  ## response
                          ## ---------------------
                          ## methods
                          ## ---------------------
